@@ -4,16 +4,16 @@ use 5.008000;
 use strict;
 use warnings;
 
-our $VERSION = '0.17_02';
+our $VERSION = '0.02';
 
 use Carp qw( croak );
 
-my %REGD_HANDLERS;
+my %REGISTERED_EVENTS;
 my %MODULES_IDX;
 
 
 sub register {
-  my $class        = shift;
+  my $self_class   = shift;
   my $module_class = shift;
   my %handlers     = @_;
 
@@ -27,17 +27,19 @@ sub register {
   my $events_idx = $MODULES_IDX{$module_class};
 
   while ( my ( $event_name, $handler ) = each %handlers ) {
-    next if exists $events_idx->{$event_name};
-
+    if ( exists $events_idx->{$event_name} ) {
+      croak qq{"$event_name" handler for "$module_class" already registered};
+    }
     $events_idx->{$event_name} = 1;
-    unless ( exists $REGD_HANDLERS{$event_name} ) {
-      $REGD_HANDLERS{$event_name} = [];
+
+    unless ( exists $REGISTERED_EVENTS{$event_name} ) {
+      $REGISTERED_EVENTS{$event_name} = [];
     }
     if ( $event_name =~ m/\:r$/ ) {
-      unshift( @{ $REGD_HANDLERS{$event_name} }, $handler );
+      unshift( @{ $REGISTERED_EVENTS{$event_name} }, $handler );
     }
     else {
-      push( @{ $REGD_HANDLERS{$event_name} }, $handler );
+      push( @{ $REGISTERED_EVENTS{$event_name} }, $handler );
     }
   }
 
@@ -45,56 +47,46 @@ sub register {
 }
 
 sub send_event {
-  my $class      = shift;
+  my $self_class = shift;
   my $event_name = shift;
-  my $cb         = pop if ref( $_[-1] ) eq 'CODE';
-  my @args       = @_;
 
   unless ( defined $event_name ) {
     croak 'Event name must be specified';
   }
 
-  unless ( exists $REGD_HANDLERS{$event_name} ) {
-    $cb->() if defined $cb;
-    return;
-  }
+  return unless exists $REGISTERED_EVENTS{$event_name};
 
-  my @handlers = @{ $REGD_HANDLERS{$event_name} };
-  if ( defined $cb ) {
-    $class->_process_async( \@handlers, \@args, $cb );
+  my @handlers = @{ $REGISTERED_EVENTS{$event_name} };
+
+  if ( ref( $_[-1] ) eq 'CODE' ) {
+    my $cb = pop @_;
+    $self_class->_async_call_handler( \@handlers, [@_], $cb );
   }
   else {
     foreach my $handler (@handlers) {
-      $handler->(@args);
+      $handler->(@_);
     }
   }
 
   return;
 }
 
-sub _process_async {
-  my $class    = shift;
-  my $handlers = shift;
-  my $args     = shift;
-  my $cb       = shift;
+sub _async_call_handler {
+  my $self_class = shift;
+  my $handlers   = shift;
+  my $args       = shift;
+  my $cb         = shift;
 
   my $handler = shift @{$handlers};
 
+  unless ( defined $handler ) {
+    $cb->();
+    return;
+  };
+
   $handler->( @{$args},
     sub {
-      my $err = shift;
-
-      if ( defined $err ) {
-        $cb->($err);
-        return;
-      }
-
-      if ( @{$handlers} ) {
-        $class->_process_async( $handlers, $args, $cb );
-        return;
-      }
-
-      $cb->();
+      $self_class->_async_call_handler( $handlers, $args, $cb );
     }
   );
 
@@ -113,17 +105,31 @@ pattern
 
   use App::Environ;
 
+  # Register handlers in your class
+
   App::Environ->register( __PACKAGE__,
-    initialize   => sub { ... },
+    initialize => sub {
+      my $cb = pop if ref( $_[-1] ) eq 'CODE';
+      my @args = @_;
+
+      # handling...
+    },
     reload       => sub { ... },
     'finalize:r' => sub { ... },
   );
 
+  # Send events from your application
+
+  # Synchronous interface
   App::Environ->send_event( 'initialize', qw( foo bar ) );
   App::Environ->send_event('reload');
-  App::Environ->send_event( 'pre_finalize:r', sub {...} );
   App::Environ->send_event('finalize:r');
 
+  # Asynchronous interface
+  App::Environ->send_event( 'initialize', qw( foo bar ), sub { ... } );
+  App::Environ->send_event( 'reload', sub { ... } );
+  App::Environ->send_event( 'finalize:r', sub { ... } );
+
 =head1 DESCRIPTION
 
 App::Environ is the simple environment to build applications using service
@@ -134,45 +140,14 @@ common resources.
 
 =head2 register( $class, \%handlers )
 
-The method registers handlers for specified events. When some event have been
-sent, registered event handlers will be processed in order in which they was
-registered. If you want that event handlers have been processed in reverse
-order, add postfix C<:r> to event name. All arguments that have been specified
-in C<send_event> method (see below) are passed to called event handler. If in
-the last argument is passed the callback, the handler must call it when
-processing will be done. If the handler was called with callback and some error
-occurred, the callback must be called with error message in first argument.
+Registers handlers for specified events. When an event have been sent, event
+handlers will be processed in order in which they was registered. If you want
+that event handlers have been processed in reverse order, add postfix C<:r> to
+event name.
 
-  App::Environ->register( __PACKAGE__,
-    initialize => sub {
-      my @args = @_;
+=head2 send_event( $event [, @args ] [, $cb->() ] )
 
-      # handling...
-    },
-  );
-
-=head2 send_event( $event [, @args ] [, $cb->( [ $err ] ) ] )
-
-Sends specified event to App::Environ. All handlers registered for this event
-will be processed. Arguments specified in C<send_event> method will be passed
-to event handlers. If the callback is passed in the last argument, event
-handlers will be processed in asynchronous mode.
-
-  App::Environ->send_event( 'initialize', qw( foo bar ) );
-
-  App::Environ->send_event( 'pre_finalize:r'
-    sub {
-      my $err = shift;
-
-      if ( defined $err ) {
-        # error handling...
-
-        return;
-      }
-
-      # success handling...
-    }
-  );
+Sends specified event. All handlers registered for this event will be processed.
 
 =head1 SEE ALSO
 
