@@ -4,16 +4,18 @@ use 5.008000;
 use strict;
 use warnings;
 
-our $VERSION = '0.17_02';
+our $VERSION = '0.02';
 
 use App::Environ;
 use Config::Processor;
+use AnyEvent;
 use Carp qw( croak );
 
-my @REGD_CONFIG_SECTIONS;
-my %CONFIG_SECTIONS_IDX;
+my @REGISTERED_SECTIONS;
 my $CONFIG;
-my $NEED_CONFIG_INIT = 1;
+my %SECTIONS_IDX;
+my $NEED_INIT = 0;
+
 
 App::Environ->register( __PACKAGE__,
   initialize   => sub { __PACKAGE__->_initialize(@_) },
@@ -27,14 +29,13 @@ sub register {
   my @config_sections = @_;
 
   foreach my $config_section (@config_sections) {
-    next if exists $CONFIG_SECTIONS_IDX{$config_section};
-
-    $CONFIG_SECTIONS_IDX{$config_section} = 1;
-    push( @REGD_CONFIG_SECTIONS, $config_section );
-
-    $NEED_CONFIG_INIT = 1;
+    next if $SECTIONS_IDX{$config_section};
+    $SECTIONS_IDX{$config_section} = 1;
+    push( @REGISTERED_SECTIONS, $config_section );
   }
 
+  $NEED_INIT = 1;
+
   return;
 }
 
@@ -48,45 +49,60 @@ sub instance {
 
 sub _initialize {
   my $class = shift;
+  my $cb = pop if ref( $_[-1] ) eq 'CODE';
 
-  return unless $NEED_CONFIG_INIT;
+  if ($NEED_INIT) {
+    $class->_load_config;
+    $NEED_INIT = 0;
+  }
 
-  $class->_load;
-  undef $NEED_CONFIG_INIT;
+  if ( defined $cb ) {
+    AE::postpone { $cb->() };
+  }
 
   return;
 }
 
 sub _reload {
   my $class = shift;
+  my $cb = pop if ref( $_[-1] ) eq 'CODE';
 
-  $class->_load;
+  $class->_load_config;
+
+  if ( defined $cb ) {
+    AE::postpone { $cb->() };
+  }
 
   return;
 }
 
 sub _finalize {
+  my $cb = pop if ref( $_[-1] ) eq 'CODE';
+
+  undef @REGISTERED_SECTIONS;
+  undef %SECTIONS_IDX;
   undef $CONFIG;
-  $NEED_CONFIG_INIT = 1;
+
+  if ( defined $cb ) {
+    AE::postpone { $cb->() };
+  }
 
   return;
 }
 
-sub _load {
+sub _load_config {
   my @config_dirs;
   if ( defined $ENV{APPCONF_DIRS} ) {
     @config_dirs = split /:/, $ENV{APPCONF_DIRS};
   }
-  my $interpolate_vars   = $ENV{APPCONF_INTERPOLATE_VARIABLES};
-  my $process_directives = $ENV{APPCONF_PROCESS_DIRECTIVES};
 
   my $config_processor = Config::Processor->new(
-    dirs => \@config_dirs,
-    $interpolate_vars ? ( interpolate_variables => $interpolate_vars ) : (),
-    $process_directives ? ( process_directives => $process_directives ) : (),
+    dirs                  => \@config_dirs,
+    interpolate_variables => $ENV{APPCONF_INTERPOLATE_VARIABLES},
+    process_directives    => $ENV{APPCONF_PROCESS_DIRECTIVES},
   );
 
-  $CONFIG = $config_processor->load(@REGD_CONFIG_SECTIONS);
+  $CONFIG = $config_processor->load(@REGISTERED_SECTIONS);
 
   return;
 }
@@ -99,13 +115,10 @@ App::Environ::Config - Configuration files processor for App::Environ
 
 =head1 SYNOPSIS
 
-  use App::Environ;
   use App::Environ::Config;
 
   App::Environ::Config->register( qw( foo.yml bar.json ) );
 
-  App::Environ->send_event('initialize');
-
   my $config = App::Environ::Config->instance;
 
 =head1 DESCRIPTION
@@ -113,14 +126,11 @@ App::Environ::Config - Configuration files processor for App::Environ
 App::Environ::Config is the configuration files processor for App::Environ.
 Allows get access to configuraton tree from different application components.
 
-The module registers in App::Environ three handlers for events C<initialize>,
-C<reload> and C<finalize:r>.
-
 =head1 METHODS
 
 =head2 register( @config_sections )
 
-The method registers configuration sections.
+Registers configuration sections.
 
 =head2 instance()
 
